# -*- coding: utf-8 -*-
# This file is used to configure and customize the LVTV application.
# As such, it is an integral part of the LVTV application and everything that applies to that application
# (contained in the lvtv.py file) also applies to this file, including licensing issues.
# Written by Alvils Berzins, licensed under the GNU General Public License version 2
# For licensing details, please see http://www.gnu.org

# The following line specifies the output file path. It can be either absolute or relative.
Output_File = "xmltv.xml"

# The following specifies the time zone
Time_Zone = "+0200"

# Sleep the specified number of seconds (>=0) after each HTTP request
Sleep_Time = 2

# The following is the list of all Latvian TV channels supported by this application.
# Currently, all 5 terrestrially broadcast channels are supported.
# Parameters for each channel are:
# 1. Channel name. This can be used, for instance, by the cruncher engine to find the channel name on the HTML page.
# 2. Channel ID. Your media center will probably search for this in the XMLTV file.
# 3. Channel ID for the engine. For instance, in the MansTV site (www.manstv.lv) a number is used for each channel.
# 4. Engine name. In the future, multiple cruncher engines could be supported, but for now, only MansTV can be here.
# 5. Visible name. To show the channel name, your media center will use this value. Change as you see fit.
# 6. Limit days. Perform crunching for the current date till current date + N days.
LVChannels = [
    ["LTV1", "ltv1.devlat.eu", "201", "MansTV", "LTV1", 3],
    ["LTV7", "ltv7.devlat.eu", "202", "MansTV", "LTV7", 3],
    ["TV3", "tv3.devlat.eu", "203", "MansTV", "TV3", 3],
    ["LNT", "lnt.devlat.eu", "204", "MansTV", "LNT", 3],
    ["TV5", "tv5.devlat.eu", "205", "MansTV", "TV5", 3],
    ["REN TV Baltic", "rentvbaltic.devlat.eu", "199", "MansTV", "REN TV Baltic", 3],
    ["TV6", "tv6.devlat.eu", "209", "MansTV", "TV6", 3],
    ["TV3+", "tv3plus.devlat.eu", "207", "MansTV", "TV3+", 3],
    ["NTV Mir", "ntvmir.devlat.eu", "106", "MansTV", "NTV Mir", 2],
    ["1. Baltijas kanals", "pirmaisbaltijaskanals.devlat.eu", "208", "MansTV", "1. Baltijas kanals", 3],
    ["RTR Planeta", "rtrplaneta.devlat.eu", "129", "MansTV", "RTR Planeta", 3],
    ["TV1000 Krievu kino", "tv1000russkojekino.devlat.eu", "231", "MansTV", "TV1000 Krievu kino", 2],
    ["TV Center International", "tvcenterinternational.devlat.eu", "104", "MansTV", "TV Center International", 2],
    ["RTVi", "rtvi.devlat.eu", "115", "MansTV", "RTVi", 2],
    ["TV1000", "tv1000.devlat.eu", "214", "MansTV", "TV1000", 3],
    ["Hallmark Channel", "hallmarkchannel.devlat.eu", "24", "MansTV", "Hallmark Channel", 3],
    ["MTV Latvija", "mtvlatvija.devlat.eu", "206", "MansTV", "MTV Latvija", 1],
    ["Eurosport", "eurosport.devlat.eu", "60", "MansTV", "Eurosport", 2],
    ["Cartoon Network", "cartoonnetwork.devlat.eu", "87", "MansTV", "Cartoon Network", 3],
    ["Discovery Channel", "discoverychannel.devlat.eu", "41", "MansTV", "Discovery Channel", 3],
    ["Extreme Sports Channel", "extremesportschannel.devlat.eu", "86", "MansTV", "Extreme Sports Channel", 1],
    ["CNN International", "cnninternational.devlat.eu", "78", "MansTV", "CNN International", 1],
    ["Nickelodeon", "nickelodeon.devlat.eu", "84", "MansTV", "Nickelodeon", 3],
    ["VH1", "vh1.devlat.eu", "68", "MansTV", "VH1", 3],
    ["ESPN Classic", "espnclassic.devlat.eu", "65", "MansTV", "ESPN Classic", 1],
    ["MCM", "mcm.devlat.eu", "97", "MansTV", "MCM", 3],
    ["PRO7", "pro7.devlat.eu", "54", "MansTV", "PRO7", 3],
    ["VIASAT Explorer", "viasatexplorer.devlat.eu", "246", "MansTV", "VIASAT Explorer", 3],
    ["Discovery World", "discoveryworld.devlat.eu", "42", "MansTV", "Discovery World", 3],
    ["Dom Kino", "domkino.devlat.eu", "142", "MansTV", "Dom Kino", 2],
    ["NASN", "nasn.devlat.eu", "66", "MansTV", "NASN", 2],
    ["Mezzo", "mezzo.devlat.eu", "74", "MansTV", "Mezzo", 3],
    ["Discovery Science", "discoveryscience.devlat.eu", "43", "MansTV", "Discovery Science", 3],
    ["Boomerang", "boomerang.devlat.eu", "88", "MansTV", "Boomerang", 3],
    ["VIASAT Sport Baltic", "viasatsportbaltic.devlat.eu", "920", "MansTV", "VIASAT Sport Baltic", 2],
    ["Discovery Travel and Living", "discoverytravelandliving.devlat.eu", "44", "MansTV", "Discovery Travel and Living", 3],
    ["VIASAT History", "viasathistory.devlat.eu", "245", "MansTV", "VIASAT History", 3],
    ["Eurosport 2", "eurosport2.devlat.eu", "61", "MansTV", "Eurosport 2", 3],
    ["TV1000 East", "tv1000east.devlat.eu", "214", "MansTV", "TV1000 East", 3],
    ["TV1000 Premium", "tv1000premium.devlat.eu", "233", "MansTV", "TV1000 Premium", 3],
    ["TV1000 Action", "tv1000action.devlat.eu", "234", "MansTV", "TV1000 Action", 3],    
    ["BBC World", "bbcworld.devlat.eu", "80", "MansTV", "BCC World", 2],
    ["Jetix", "jetix.devlat.eu", "90", "MansTV", "Jetix", 3],
    ["Disney Channel", "disneych.devlat.eu", "905", "MansTV", "Disney channel", 3],
    ["Playhouse Disney", "playhousedisney.devlat.eu", "906", "MansTV", "Playhouse Disney", 3],
    ["TOON Disney", "toondisney.devlat.eu", "907", "MansTV", "Toon Disney", 3],
    ["E!", "e.devlat.eu", "100", "MansTV", "E!", 2],    
    ["VIASAT Sport", "viasatsport.devlat.eu", "922", "MansTV", "VIASAT Sport", 3],
    ["VIASAT Golf", "viasatgolf.devlat.eu", "923", "MansTV", "VIASAT Golf", 3],
    ["Private Spice", "privatespice.devlat.eu", "980", "MansTV", "Private Spice", 3],
    ["Playboy", "playboy.devlat.eu", "985", "MansTV", "Playboy", 3],
    ["Travel", "travel.devlat.eu", "48", "MansTV", "Travel", 3],
]

# This function generates the URL to fetch data from. As you see, currently only the MansTV engine is supported.
def LVGetURL(Channel, Year, Month, Day):
    Engine = LVChannels[Channel][3]
    if Engine == "MansTV":
        # Engine is MansTV, so all further processing is done according to this
        return "http://www.manstv.lv/?page=chn&cgid=1&cid="+LVChannels[Channel][2]+"&d="+str(Day)+"&m="+str(Month)+"&y="+str(Year)+"&clicktrack=chn,"+LVChannels[Channel][2]
    # Unknown engine
    return "Error - no engine defined."

