# -*- coding: utf-8 -*-
# LVTV - an application to fetch XMLTV data for Latvian TV channels
# Written by Alvils Berzins, licensed under the GNU General Public License version 2.
# For licensing details, please see http://www.gnu.org/
# Version 1.1.4, 25.12.2010
#
# For additional information see the README file.
#
import urllib, datetime, sgmllib, string, re, time
from config import *
VERSION = "1.1.4"

# I found this class somewhere on the Web, so all credits go to its author.
# Unfortunately, I wasn't able to find the author, so he/she remains unknown. :(
# If you happen to know him/her, please let me know: alvils [at] devlat [dot] eu
class StrippingParser(sgmllib.SGMLParser):

    # These are the HTML tags that we will leave intact
    valid_tags = ('input')

    from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
    
    def __init__(self):
        sgmllib.SGMLParser.__init__(self)
        self.result = ""
        self.endTagList = [] 
        
    def handle_data(self, data):
        if data:
            self.result = self.result + data

    def handle_charref(self, name):
        self.result = "%s&#%s;" % (self.result, name)
        
    def handle_entityref(self, name):
        if self.entitydefs.has_key(name): 
            x = ';'
        else:
            # this breaks unstandard entities that end with ';'
            x = ''
        self.result = "%s&%s%s" % (self.result, name, x)
    
    def unknown_starttag(self, tag, attrs):
        """ Delete all tags except for legal ones """
        if tag in self.valid_tags:       
            self.result = self.result + '<' + tag
            for k, v in attrs:
                if string.lower(k[0:2]) != 'on' and string.lower(v[0:10]) != 'javascript':
                    self.result = '%s %s="%s"' % (self.result, k, v)
            endTag = '</%s>' % tag
            self.endTagList.insert(0,endTag)    
            self.result = self.result + '>'
                
    def unknown_endtag(self, tag):
        if tag in self.valid_tags:
            self.result = "%s</%s>" % (self.result, tag)
            remTag = '</%s>' % tag
            try:
		self.endTagList.remove(remTag)
	    except:
		print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | TEMPORARY FIX: this HTML has problems with tags. Trying to ignore that..."

    def cleanup(self):
        """ Append missing closing tags """
        for j in range(len(self.endTagList)):
                self.result = self.result + self.endTagList[j]    
        
# This is the "cruncher" function of the MansTV web site. It takes the following arguments:
# Channel - index of the LVChannels array (see config.py)
# URL - URL generated by the LVGetURL function (see config.py)
# yy, mm, dd - year, month and day of the programme
# yy2, mm2, dd2 - reserved for future use. Stupid, but that's the way it is. :) :) :)
# This function processes one day's programme of a single channel and
# returns its results in the global DailyProgramme list.
def ProcessMansTV(Channel, URL, yy, mm, dd, yy2, mm2, dd2):
    global DailyProgramme
    # Fetch the HTML data
    print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Fetching data"
    stream = urllib.urlopen(URL)
    RawData = stream.read()
    stream.close()
    # Sleep the specified number of seconds. Sometimes programme servers get overloaded by an endless stream of requests...
    if Sleep_Time > 0:
	print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Waiting " + str(Sleep_Time) + " second(s)"
	time.sleep(Sleep_Time)
    print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Crunching"
    # Eliminate most HTML tags
    parser = StrippingParser()
    parser.feed(RawData)
    parser.close()
    parser.cleanup()
    StripData = parser.result
    # Find the beginning of any meaningful data
    ChannelID = LVChannels[Channel][0]
    Posit = string.find(StripData, "%02d.%02d.%02d"%(dd,mm,yy))
    StripData = StripData[Posit:]
    Posit = 0
    Finished = 0
    LastProgTime=""
    # Now, we have to find the "<input type="hidden"". Before that is the time,
    # in the tag is the comment, and after that is the name of the programme.
    while Finished == 0:
        ProgTime = ""
        ProgComment = ""
        RawComment = ""
        ProgName = ""
        while StripData[Posit:Posit+20] != '<input type="hidden"':
            Posit = Posit + 1
	    if Posit > len(StripData):
		print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Bad data found (1). Skipping."
		return
        ProgTime = StripData[Posit-5]+StripData[Posit-4]+StripData[Posit-2]+StripData[Posit-1]
        # Now that we have the ProgTime, parse the <input> tag to get the comment.
        while StripData[Posit:Posit+7] != 'value="':
            Posit = Posit + 1
        Posit = Posit + 7
        while StripData[Posit:Posit+2] != '">':
            RawComment = RawComment + StripData[Posit]
            Posit = Posit + 1
        Posit = Posit + 2
        while StripData[Posit:Posit+7] != '<p></p>':
	    # Added in version 1.1.2 to avoid "bad" or empty chans since the Summer solstice of 2010. :)
	    if Posit >= len(StripData):
		print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Bad data found (2). Skipping."
		return
            ProgName = ProgName + StripData[Posit]
            Posit = Posit + 1
        Posit = Posit + 7
        # Now, process RawComment to get the "real" comment
        RawPos = 0
        while RawComment[RawPos:RawPos+2] != ';;':
            RawPos = RawPos + 1
        RawPos = RawPos + 2
        while RawComment[RawPos:RawPos+2] != ';;':
            ProgComment = ProgComment + RawComment[RawPos]
            RawPos = RawPos + 1
        while (StripData[Posit] == ' ') or (StripData[Posit] == '\n') or (StripData[Posit] == '&') or (StripData[Posit] == 'I'):
	    if StripData[Posit] == '&':
		if StripData[Posit:Posit+8] == '&rsaquo;':
			Posit = Posit + 8
		if StripData[Posit:Posit+6] == '&nbsp;':
			Posit = Posit + 6
	    else:
		# Added as of version 1.1.0. A very specific condition where IMDB data is "Kids morning 3".
		# It's a show on Latvian TV3. As it ends with a number, this fucks up the parser.
#   		if StripData[Posit:Posit+20] == 'IMDB: Kids morning 3':
#    		        Posit = Posit + 20
		if StripData[Posit] == 'I':
			# Fuck. The IMDB data... :(
		 	# The good thing is that for some channels, the IMDB metadata is available.
			# The bad thing is that we have to eliminate it, as we don't need that in our XMLTV data,
			# at least not for this version of the program...
			Posit = Posit + 1
			while StripData[Posit] not in ['0', '1', '2']:
				Posit = Posit + 1
			if StripData[Posit+2] == '.':
				Posit = Posit - 2
	    	else:
            		Posit = Posit + 1
	# Added in version 1.1.2. A link to more options can now be possible.
	if StripData[Posit:Posit+2]=="R:":
	    while StripData[Posit:Posit+4]!="Vair":
		Posit=Posit+1
	    Posit=Posit+7
	    while StripData[Posit] not in ['0','1','2']:
		Posit=Posit+1
		if Posit > len(StripData):
			print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Info: end-of-day found."
			Finished = 1
	if Finished == 0:
		# A hack of version 1.1.4
		if StripData[Posit:Posit+3]=='<i>':
		    while StripData[Posit:Posit+4]!='</i>':
			Posit = Posit + 1
		    Posit = Posit + 26
		if StripData[Posit]=="V":
		    Posit=Posit+7
		if StripData[Posit]=='&':
		    Posit=Posit+35
		    if StripData[Posit]=='\n':
			Posit=Posit+2
		if StripData[Posit]=='\n':
		    Posit = Posit+2
		# ---
        if StripData[Posit] not in ['0','1','2']:
            Finished = 1
		# Added as of version 1.0.1:
	# Parse ProgName additionally. If "> is met in the name, then the "real" name comes after that.
	# This is needed due to a new "fucking" feature in MansTV starting from April 13, 2009...
	NamePosit = 0;
	while NamePosit < len(ProgName):
	    if ProgName[NamePosit:NamePosit+2] == '">':
		ProgName = ProgName[NamePosit+2:]
	    NamePosit = NamePosit + 1
	# Parse the ProgComment and eliminate any possible HTML tags
	parser = StrippingParser()
	parser.feed(ProgComment)
	parser.close()
	parser.cleanup()
	ProgComment = parser.result
	# Added in version 1.0.2 - needed to keep the XML healthy
	ProgComment = ProgComment.replace('&', '&amp;')
	# Added in version 1.1.3
	ProgComment = ProgComment.replace('<', '&lt;')
	ProgComment = ProgComment.replace('>', '&gt;')
	# Added in version 1.0.4 - needed to keep the XML healthy
	# Updated in version 1.1.2.
	ProgName = ProgName.replace('</i>', '')
	ProgName = ProgName.replace('&nbsp;', ' ')
	ProgName = ProgName.replace('&', '&amp;')
	# Added in version 1.1.3
	ProgName = ProgName.replace('<', '&lt;');
	ProgName = ProgName.replace('>', '&gt;');
	# Another addition is the check whether there are no items with identical timings.
	ValidItem = 0
	if LastProgTime=="":
	    ValidItem = 1
	if LastProgTime < ProgTime:
	    ValidItem = 1
	if ValidItem == 1:
    	    DailyProgramme.append([yy,mm,dd,ProgTime,ProgName,ProgComment,LVChannels[Channel][1]])
	    LastProgTime = ProgTime

# This functions adds stop time to the programme.
# Added as of version 1.1.0.
def AddStopTime():
    for Chan in reversed(GlobalProgramme):
	StopTime = '235959'
	for Item in reversed(Chan):
	    if StopTime == '235959':
		StopYY = Item[0]
		StopMM = Item[1]
		StopDD = Item[2]
	    Item.append(StopTime)
	    Item.append(StopYY)
	    Item.append(StopMM)
	    Item.append(StopDD)
	    StopTime = Item[3]
	    StopYY = Item[0]
	    StopMM = Item[1]
	    StopDD = Item[2]
	    StopHr = string.atoi(StopTime[0]+StopTime[1])
	    StopMn = string.atoi(StopTime[2]+StopTime[3])
	    StopMn = StopMn - 1
	    if StopMn == -1:
		StopHr = StopHr - 1
		StopMn = 59
		if StopHr == -1:
		    StopHr = 23
	    StopTime = ''
	    if StopHr < 10:
		StopTime = StopTime + '0'
	    StopTime = StopTime + '%d'%StopHr
	    if StopMn < 10:
		StopTime = StopTime + '0'
	    StopTime = StopTime + '%d'%StopMn
	    StopTime = StopTime +'59'
		

# This function outputs the XMLTV file from the data collected.
def WriteXML():
    global GlobalProgramme, Output_File, LVChannels
    Out = open(Output_File, "wt")
    Out.write('<?xml version="1.0" encoding="utf-8" ?>\n')
    Out.write('<tv generator-info-name="LVTV/');
    Out.write(VERSION);
    Out.write('" generator-info-url="http://lvtv.devlat.eu/">\n')
    for Chan in LVChannels:
        Out.write('<channel id="'+Chan[1]+'">\n')
        Out.write('<display-name lang="lv">'+Chan[4]+'</display-name>\n')
        Out.write('</channel>\n')
    for Chan in GlobalProgramme:
        for Item in Chan:
            ProgStart = "%02d%02d%02d"%(Item[0],Item[1],Item[2]) + Item[3] + '00 ' + Time_Zone
	    ProgStop = "%02d%02d%02d"%(Item[8],Item[9],Item[10]) + Item[7] + ' ' + Time_Zone
            Out.write('<programme start="'+ProgStart+'" stop="'+ProgStop+'" channel="'+Item[6]+'">\n')
            Out.write('<title lang="lv">'+Item[4]+'</title>\n')
            if Item[5]!='':
                Out.write('<desc lang="lv">'+Item[5]+'</desc>\n')
            Out.write('</programme>\n')
    Out.write('</tv>\n\n')
    Out.close

print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | LVTV version " + VERSION
print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Preparing"
    
DailyProgramme = []
ChannelProgramme = []
GlobalProgramme = []
today = datetime.date.today()
one_day = datetime.timedelta(days=1)

day1 = today - one_day
day2 = today
day3 = today + one_day
day4 = today + one_day + one_day
day5 = today + one_day + one_day + one_day

# The main loop that processes all the channels. Every channel can have its own cruncher engine.
Current = 0
for Channel in LVChannels:
    print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Processing " + LVChannels[Current][4]
    ChannelProgramme = []
    # Currently, MansTV is the only supported cruncher engine.
    if LVChannels[Current][3] == "MansTV":
	# I know, this looks stupid. But it works, so don't scream. :)
	print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Using engine MansTV"
        yy1 = day1.year
        mm1 = day1.month
        dd1 = day1.day
        yy2 = day2.year
        mm2 = day2.month
        dd2 = day2.day
        yy3 = day3.year
        mm3 = day3.month
        dd3 = day3.day
        yy4 = day4.year
        mm4 = day4.month
        dd4 = day4.day
        yy5 = day5.year
        mm5 = day5.month
        dd5 = day5.day
	# Prepare the URL and crunch its data for yesterday, today, the day after that, another day after that,
	# and yet another day after that... :)
	print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Processing yesterday"
        URL = LVGetURL(Current, yy1, mm1, dd1)
        ProcessMansTV(Current, URL, yy1, mm1, dd1, yy1, mm1, dd1)
        for Item in DailyProgramme:
            ChannelProgramme.append(Item)
        DailyProgramme = []
	print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Processing today"
        URL = LVGetURL(Current, yy2, mm2, dd2)
        ProcessMansTV(Current, URL, yy2, mm2, dd2, yy2, mm2, dd2)
        for Item in DailyProgramme:
            ChannelProgramme.append(Item)        
        DailyProgramme = []
	if LVChannels[Current][5] > 0:
		print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Processing today+1"
	        URL = LVGetURL(Current, yy3, mm3, dd3)
	        ProcessMansTV(Current, URL, yy3, mm3, dd3, yy3, mm3, dd3)
	        for Item in DailyProgramme:
	            ChannelProgramme.append(Item)        
	        DailyProgramme = []
	if LVChannels[Current][5] > 1:
		print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Processing today+2"
                URL = LVGetURL(Current, yy4, mm4, dd4)
	        ProcessMansTV(Current, URL, yy4, mm4, dd4, yy4, mm4, dd4)
	        for Item in DailyProgramme:
	            ChannelProgramme.append(Item)        
	        DailyProgramme = []
	if LVChannels[Current][5] > 2:
		print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Processing today+3"
	        URL = LVGetURL(Current, yy5, mm5, dd5)
	        ProcessMansTV(Current, URL, yy5, mm5, dd5, yy5, mm5, dd5)
	        for Item in DailyProgramme:
	            ChannelProgramme.append(Item)
	        DailyProgramme = []
    print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Creating the global programme"
    GlobalProgramme.append(ChannelProgramme)
    Current = Current + 1

# Now, just output LVChannels and GlobalProgramme information to an XML file...
print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Calculating programme stop times"
AddStopTime()
print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Writing the final file"
WriteXML()
print datetime.datetime.now().strftime("%d.%m.%Y %H:%M:%S") + " | Done"
